{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "# Vier op een rij: de klasse `Board`"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Voor deze opgave moet je met een nieuw bestand `wk10ex2.py` beginnen.\n",
    "\n",
    "Vier op een rij is een variant van Boter, kaas en eieren dat gespeeld wordt op een rechthoekig bord van 7 bij 6:\n",
    "\n",
    "![Vier op een rij](images/11/game_b2.png)\n",
    "\n",
    "Het spel wordt gespeeld door twee spelers, die om de beurt een zet doen, waarbij beide spelers proberen om vier stenen op een verticale, horizontale of diagonale rij te krijgen. Eén voorwaarde bij het spelen is dat, omdat het bord verticaal opgesteld staat, de stenen niet zomaar overal geplaatst kunnen worden. Een stee mag alleen maar bovenaan een bestaande kolom stenen gespeeld worden (of het mag een nieuwe kolom stenen beginnen).\n"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## De klasse `Board`\n",
    "\n",
    "In deze opgave ga je een klasse genaamd `Board` maken die sommige eigenschappen van het spel Vier op een rij implementeert. De klasse `Board` heeft drie instantievariabelen: er is een tweedimensionale lijst (een lijst van lijsten) met karakters die het spelbord voorstelt, en twee variabelen die het aantal rijen en kolommen op het bord bijhouden (6 rijen en 7 kolommen is standaard, maar je klasse `Board` kan elk formaat bord aan).\n",
    "\n",
    "Zelfs als we onbeperkt grote borden toestaan blijft de regel echter bestaan dat je vier stenen op een rij nodig hebt om te winnen. Toegegeven, dit maakt het wel lastig om het spel te winnen op een bord van 3 bij 3..."
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Je klasse `Board` heeft minstens drie instantievariabelen:\n",
    "\n",
    "* Een variabele met de naam `data` waarin de tweedimensionale array (lijst van lijsten) met het spelbord wordt opgeslagen\n",
    "* Een variabele `height` waarin het aantal rijen op het bord wordt opgeslagen\n",
    "* Een variabele `width` waarin het aantal kolommen op het bord wordt opgeslagen\n",
    "\n",
    "Merk op dat de tweedimensionale lijst een tweedimensionale lijst van *karakters* is; dit zijn gewoon strings van lengte 1. Je moet een leeg veld laten voorstellen door `' '`, een spatieteken; niet de lege string. Je moet de stenen van speler X laten voorstellen door een `'X'` (de hoofdletter x) en je moet de stenen van speler O laten voorstellen door een `'O'` (de hoofdletter o)."
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    ":::{admonition} Waarschuwing\n",
    ":class: danger\n",
    "\n",
    "Een **heel** moeilijk te vinden bug treedt op als je het ***nulteken*** `'0'` gebruikt in plaats van het teken `'O'` (hoofdletter o) om één van de spelers voor te stellen. Het probleem treedt op als door jou, of tijdens het nakijken, de waardes in het bord vergeleken worden met de verkeerde karakters! Zorg ervoor dat je consistent bent in het gebruik van de hoofdletter o.\n",
    ":::"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Je moet een aantal methodes implementeren in de klasse `Board`. Vergeet niet de tips over hoe je ze kan testen te gebruiken nadat je ze geschreven hebt! De eerste twee methodes zijn in het college behandeld en zijn hieronder gegeven zodat je ze kan kopiëren en plakken."
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "```python\n",
    "class Board:\n",
    "    \"\"\"A data type representing a Connect-4 board\n",
    "       with an arbitrary number of rows and columns.\n",
    "    \"\"\"\n",
    "\n",
    "    def __init__(self, width, height):\n",
    "        \"\"\"Construct objects of type Board, with the given width and height.\"\"\"\n",
    "        self.width = width\n",
    "        self.height = height\n",
    "        self.data = [[' ']*width for row in range(height)]\n",
    "\n",
    "        # We hoeven niets terug te geven vanuit een constructor!\n",
    "\n",
    "    def __repr__(self):\n",
    "        \"\"\"This method returns a string representation\n",
    "           for an object of type Board.\n",
    "        \"\"\"\n",
    "        s = ''                          # de string om terug te geven\n",
    "        for row in range(0, self.height):\n",
    "            s += '|'\n",
    "            for col in range(0, self.width):\n",
    "                s += self.data[row][col] + '|'\n",
    "            s += '\\n'\n",
    "\n",
    "        s += (2*self.width + 1) * '-'   # onderkant van het bord\n",
    "\n",
    "        # hier moeten de nummers nog onder gezet worden\n",
    "\n",
    "        return s       # het bord is compleet, geef het terug\n",
    "\n",
    "```"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Implementeer vervolgens de volgende methodes in je klasse `Board`. Vergeet niet ze allemaal te testen nadat je ze geschreven hebt; het is veel makkelijker om ze elk onafhankelijk te testen dan om te proberen een grote verzameling methodes tegelijk te debuggen."
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### De constructror `__init__`\n",
    "\n",
    "De klasse `Board` heeft een constructor `__init__(self, width, height)` die twee argumenten meekrijgt. (Onthoud dat `self` verwijst naar het object dat aangemaakt wordt en dat het niet expliciet wordt meegegeven aan de constructor.) De constructor krijgt een aantal kolommen en rijen mee (7 en 6 zijn de standaard in Vier op een rij, maar ons datatype kan alle bordformaten aan). De constructor stelt de waardes van de instantievariabelen van het object in. Het maakt bovendien de tweedimensionale lijst aan die gebruikt wordt om het bord op te slaan. Je maakt hiervoor een lijst van lijsten *op een manier die erg lijkt op die van het practicum over Game of Life*.\n",
    "\n",
    "Merk op dat `self.data` niet alle karakters bevat die gebruikt worden om het bord af te drukken; alleen de karakters die nodig zijn om het spel te spelen. De extra karakters worden gegenereerd in de methode `__repr__`."
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### De methode `__repr__`\n",
    "\n",
    "De methode `__repr__(self)` geeft een string`repr`esentatie terug van het `Board`-object dat de methode aanroept. Elke \"steen\" neemt één teken in beslag, en alle kolommen worden gescheiden door verticale lijnen (`|`). De kolommen worden aan de onderkant gelabeld. Hier is een voorbeeld voor een bord met 6 rijen en 7 kolommen, 6 bij 7 dus (zie de onderstaande code voor meer details...)\n",
    "\n",
    "```text\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "---------------\n",
    " 0 1 2 3 4 5 6\n",
    "```"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Om te zorgen dat alles netjes onder elkaar blijft staan, moet de kolomnummering \"modulo 10\" gedaan worden,\n",
    "zoals te zien in dit grotere voorbeeld met 5 rijen en 15 kolommen (5 bij 15):\n",
    "\n",
    "```\n",
    "| | | | | | | | | | | | | | | |\n",
    "| | | | | | | | | | | | | | | |\n",
    "| | | | | | | | | | | | | | | |\n",
    "| | | | | | | | | | | | | | | |\n",
    "| | | | | | | | | | | | | | | |\n",
    "-------------------------------\n",
    " 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4\n",
    "```"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    ":::{admonition} Eén string, meerdere regels?\n",
    ":class: tip\n",
    "\n",
    "De string `'\\n'` stelt het *newline*-karakter voor in Python. Je kan dus meerdere regels in een enkele string zetten door `'\\n'` toe te voegen. Bijvoorbeeld met\n",
    "\n",
    "```ipython\n",
    "In [1]: s = 'Dit is de bovenste regel.'\n",
    "In [2]: s += '\\n'\n",
    "In [3]: s += 'Dit is de tweede regel!\\n'\n",
    "In [4]: print(s)\n",
    "Dit is de bovenste regel.\n",
    "Dit is de tweede regel!\n",
    "```\n",
    "\n",
    "Merk op dat `'\\n'` ook *in* een string gebruikt kan worden. In dit geval voegt het een extra lege regel toe aan het einde van de string.\n",
    ":::"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### De methode `add_move`\n",
    "\n",
    "De methode `add_move(self, col, ox)` krijgt twee argumenten mee: het eerste, `col`, geeft de kolom aan waaraan de steen moet worden toegevoegd. Het tweede argument, `ox`, is een string van 1 karakter die de steen aangeeft die aan het bord moet worden toegevoegd. Dat wil zeggen, `ox` moet `'X'` of `'O'` zijn (hoofdletter O dus, niet nul)."
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    ":::{admonition} Onthoud dat de stenen van bovenaf in het bord vallen!\n",
    ":class: notice\n",
    "\n",
    "Je code moet dus het geschikte rijnummer vinden die beschikbaar is in kolom `col` en de steen in die rij toevoegen. Je hoeft in `add_move` **niet** te controleren of `col` een geldig kolomnummer is en of er ruimte is in kolom `col`. Die controles zijn wel belangrijk: ze worden in een toekomstige methode, `allows_move`, gedaan.\n",
    ":::"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Hier is een reeks zetten om `add_move` mee te testen; probeer ze uit!\n",
    "\n",
    "```ipython\n",
    "In [1]: b = Board(7, 6)\n",
    "In [2]: b.add_move(0, 'X')\n",
    "In [3]: b.add_move(0, 'O')\n",
    "In [4]: b.add_move(0, 'X')\n",
    "In [5]: b.add_move(3, 'O')\n",
    "In [6]: b.add_move(4, 'O')  # Valsspelen door O opnieuw te laten zetten!\n",
    "In [7]: b.add_move(5, 'O')\n",
    "In [8]: b.add_move(6, 'O')\n",
    "In [9]: print(b)\n",
    "```\n",
    "\n",
    "En wat hieruit moet komen is:\n",
    "\n",
    "```text\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "|X| | | | | | |\n",
    "|O| | | | | | |\n",
    "|X| | |O|O|O|O|\n",
    "---------------\n",
    " 0 1 2 3 4 5 6\n",
    "```"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### De methode `clear`\n",
    "\n",
    "De methode `clear(self)` moet het aanroepende bord leeg maken; er valt hier verder weinig over te zeggen, maar de methode is wel nuttig!"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### De methode `set_board`\n",
    "\n",
    "De methode `set_board(self, move_string)` is erg handig om snel een bord te maken om de methode `wins_for` te testen in het volgende deel. Hier is de code die we voor `set_board` gebruikt hebben; voeg dit toe aan\n",
    "je klasse `Board`, het maakt het testen veel simpeler!\n",
    "\n",
    "```python\n",
    "def set_board(self, move_string):\n",
    "    \"\"\"Accepts a string of columns and places\n",
    "       alternating checkers in those columns,\n",
    "       starting with 'X'.\n",
    "\n",
    "       For example, call b.set_board('012345')\n",
    "       to see 'X's and 'O's alternate on the\n",
    "       bottom row, or b.set_board('000000') to\n",
    "       see them alternate in the left column.\n",
    "\n",
    "       move_string must be a string of one-digit integers.\n",
    "    \"\"\"\n",
    "    next_checker = 'X'   # we starten door een 'X' te spelen\n",
    "    for col_char in move_string:\n",
    "        col = int(col_char)\n",
    "        if 0 <= col <= self.width:\n",
    "            self.add_move(col, next_checker)\n",
    "        if next_checker == 'X':\n",
    "            next_checker = 'O'\n",
    "        else:\n",
    "            next_checker = 'X'\n",
    "```"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### De methode `allows_move`\n",
    "\n",
    "De methode `allows_move(self, col)` methode moet `True` teruggeven als in het aanroepende `Board`-object een zet in kolom `col` *wel** geldig is. Ze moet `False` teruggeven als kolom `col` geen geldig kolomnummer is voor het aanroepende object. Ze moet ook `False` teruggeven als kolom `col` vol is. De methode moet dus controleren dat `col` in het bereik van 0 tot de laatste kolom valt *en* dat er nog ruimte over is in de kolom!\n",
    "\n",
    "Hier is een voorbeeldreeks om mee te testen; probeer hem uit!\n",
    "\n",
    "```ipython\n",
    "In [1]: b = Board(2, 2)\n",
    "\n",
    "In [2]: b\n",
    "Out[2]:\n",
    "| | |\n",
    "| | |\n",
    "-----\n",
    " 0 1\n",
    "\n",
    "In [3]: b.add_move(0, 'X')\n",
    "\n",
    "In [4]: b.add_move(0, 'O')\n",
    "\n",
    "In [5]: b\n",
    "Out[5]:\n",
    "|O| |\n",
    "|X| |\n",
    "-----\n",
    " 0 1\n",
    "\n",
    "In [6]: b.allows_move(-1)\n",
    "Out[6]: False\n",
    "\n",
    "In [7]: b.allows_move(0)\n",
    "Out[7]: False\n",
    "\n",
    "In [8]: b.allows_move(1)\n",
    "Out[8]: True\n",
    "\n",
    "In [9]: b.allows_move(2)\n",
    "Out[9]: False\n",
    "```"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### De methode `is_full`\n",
    "\n",
    "De methode `is_full(self)` moet `True` teruggeven als het aanroepende `Board`-object volledig gevuld is met stenen. Ze moet anders `False` teruggeven. Merk op dat je `allows_move` kan gebruiken om deze methode heel kort te schrijven! Als je niet buitengewoon geduldig bent is het handiger om dit op kleinere borden te testen!\n",
    "\n",
    "Hier is een voorbeeldreeks om mee te testen; hij gebruikt de methode `set_board` die je hierboven kan vinden.\n",
    "\n",
    "```ipython\n",
    "In [1]: b = Board(2, 2)\n",
    "\n",
    "In [2]: b.is_full()\n",
    "Out[2]: False\n",
    "\n",
    "In [3]: b.set_board('0011')\n",
    "\n",
    "In [4]: b\n",
    "Out[4]:\n",
    "|O|O|\n",
    "|X|X|\n",
    "-----\n",
    " 0 1\n",
    "\n",
    "In [5]: b.is_full()\n",
    "Out[5]: True\n",
    "```"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### De methode `del_move`\n",
    "\n",
    "De methode `del_move(self, col)` moet het omgekeerde doen van `add_move`. Ze moet de bovenste steen van kolom `col` verwijderen. Als de kolom leeg is moet `del_move` *niets* doen. Deze functie lijkt nu niet handig, maar het zal *erg* handig zijn als je je eigen computerspeler voor Vier op een rij wilt implementeren...\n",
    "\n",
    "Hier is een voorbeeldreeks om mee te testen:\n",
    "\n",
    "```ipython\n",
    "In [1]: b = Board(2, 2)\n",
    "\n",
    "In [2]: b.set_board('0011')\n",
    "\n",
    "In [3]: b.del_move(1)\n",
    "\n",
    "In [4]: b.del_move(1)\n",
    "\n",
    "In [5]: b.del_move(1)\n",
    "\n",
    "In [6]: b.del_move(0)\n",
    "\n",
    "In [7]: b\n",
    "Out[7]:\n",
    "| | |\n",
    "|X| |\n",
    "-----\n",
    " 0 1\n",
    "```"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### De methode `wins_for`\n",
    "\n",
    "De methode `wins_for(self, ox)` controleert of iemand het spel gewonnen heeft. Het argument `ox` van deze methode is een steen van 1 karakter: `'X'` of `'O'`. De methode moet `True` teruggeven als er vier stenen van het type `ox` op een rij zijn op het bord. Ze moet anders `False` teruggeven."
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    ":::{admonition} Let op\n",
    ":class: danger\n",
    "\n",
    "Je moet controleren of de speler horizontaal, verticaal of diagonaal gewonnen heeft (en er zijn twee verschillende richtingen voor een diagonale overwinning).\n",
    "\n",
    "Eén mogelijke manier om dit aan te pakken is om je functies uit week prorammeren 1 week 5 te gebruiken in een paar geneste lussen om alle mogelijke manieren om te winnen te controleren. [Hier is een pagina met een aanzet voor deze strategie](../support/wins_for.md)\n",
    ":::"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    ":::{admonition} Waarschuwing\n",
    ":class: warning\n",
    "\n",
    "De in-een-rij-functies moeten ***buiten*** de klasse staan; ze zijn globaal, en geen onderdeel van de klasse `Board`!\n",
    ":::"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Dit is een belangrijke methode die goed getest moet worden! Hier is een voorbeeldreeks om mee te testen:\n",
    "\n",
    "```ipython\n",
    "In [1]: b = Board(7, 6)\n",
    "\n",
    "In [2]: b.set_board('00102030')\n",
    "\n",
    "In [3]: b.wins_for('X')\n",
    "Out[3]: True\n",
    "\n",
    "In [4]: b.wins_for('O')\n",
    "Out[4]: True\n",
    "\n",
    "In [5]: b = Board(7, 6)\n",
    "\n",
    "In [6]: b.set_board('23344545515')\n",
    "\n",
    "In [7]: b\n",
    "Out[7]:\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | | | |X| |\n",
    "| | | | |X|X| |\n",
    "| | | |X|X|O| |\n",
    "| |O|X|O|O|O| |\n",
    "---------------\n",
    " 0 1 2 3 4 5 6\n",
    "\n",
    "In [8]: b.wins_for('X')  # diagonaal\n",
    "Out[8]: True\n",
    "\n",
    "In [9]: b.wins_for('O')\n",
    "Out[9]: False\n",
    "```"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### De methode `host_game`\n",
    "\n",
    "De methode `host_game(self)` voegt alles samen tot het complete spel Vier op een rij. Ze moet de gebruiker Vier op een rij laten spelen, en hiervoor de bovenstaande methodes gebruiken. In het bijzonder moeten `'X'` (die altijd als eerste is) en `'O'` (die altijd als tweede is) afwisselend aan de beurt zijn. De methode moet de gebruiker (door middel van de functie `input`) vragen een kolomnummer te selecteren voor elke zet. Hieronder kan je een voorbeeld zien van hoe deze interactie moet werken, maar hier zijn een paar belangrijke punten om in de gaten te houden:\n",
    "\n",
    "* Deze methode moet elke keer het bord afdrukken voordat de zet gevraagd wordt.\n",
    "* Het is handig om een grote `while`-lus te gebruiken om het spel te structureren. `'X'` is eerst aan de beurt en daarna `'O'`. We stellen voor dat je beurt van zowel `'X'` als `'O'` in de body van de lus zet. Op die manier speelt één iteratie twee zetten in Vier op een rij.\n",
    "* Je kan ook een oneindige lus `while True:` gebruiken en dan ergens in de body van de lus `break` gebruiken als het spel eindigt.\n",
    "\n",
    "Controleer na elke `input` of de gekozen kolom geldig is. Deze methode moet dus ongeldige zetten, zowel als de kolom niet bestaat of als deze vol is, herkennen, en de gebruiker om een andere kolom vragren. Je hoeft echter niet te controleren of de invoer van de gebruiker een integer is; je mag ervan uitgaan dat dit altijd zo is."
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Je kan als richtlijn voor het voorkomen van *ongeldige* zetten onderstaande kleine lus gebruiken:\n",
    "\n",
    "```python\n",
    "users_col = -1\n",
    "while not self.allows_move(users_col):\n",
    "    users_col = int(input(\"Kies een kolom: \"))\n",
    "```\n",
    "\n",
    "Bovenstaande code vraagt de gebruiker simpelweg net zo lang om een geldig kolomnummer tot hij er één ontvangt.\n",
    "\n",
    "Let op de elegante constructie `while not self.allows_move(...)`. Dit is exact hetzelfde als `while self.allows_move(...) == False`. Het beschrijft echter heel mooi het idee om door te gaan zolang de zet NIET geldig is\".\n",
    "\n",
    "De methode `host_game` moet elke steen in de door de gebruiker gekozen (geldige!) kolom zetten. Gebruik hiervoor `add_move`! Daarna moet ze controleren of die speler gewonnen heeft; als dat zo is, moet er een bericht worden afgedrukt en moet de lus afgebroken worden met `break`. Ook moet je controleren of het bord nu vol is; als dat zo is, moet er een bericht worden afgedrukt en moet de lus afgebroken worden met `break`.\n",
    "\n",
    "Als het spel dus afgelopen is voor één van deze redenen, moet het spel stoppen, het bord moet nog een laatste keer worden afgedrukt en het programma moet vertellen wie er gewonnen heeft (of dat het een gelijkspel was.) Merk op dat je `break` kan gebruiken om uit een lus te komen; zelfs als die `break` in een `if`-`else`-statement staat.\n",
    "\n",
    "Als het spel niet over is, moet de andere speler gevraagd worden een zet te doen, en zo verder...\n",
    "\n",
    "Vergeet niet de methode te testen door het spel een paar keer te spelen (met elke mogelijke uitkomst...)!\n",
    "\n",
    "Hier is een voorbeelduitvoer, om een idee te geven van de invoer en uitvoer:\n",
    "\n",
    "```ipython\n",
    "In [1]: b = Board(7, 6)\n",
    "\n",
    "In [2]: b.host_game()\n",
    "\n",
    "Welkom bij Vier op een rij!\n",
    "\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "---------------\n",
    " 0 1 2 3 4 5 6\n",
    "\n",
    "Keuze van X:  3\n",
    "\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | |X| | | |\n",
    "---------------\n",
    " 0 1 2 3 4 5 6\n",
    "\n",
    "Keuze van O:  4\n",
    "\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | |X|O| | |\n",
    "---------------\n",
    " 0 1 2 3 4 5 6\n",
    "\n",
    "Keuze van X:  2\n",
    "\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | |X|X|O| | |\n",
    "---------------\n",
    " 0 1 2 3 4 5 6\n",
    "\n",
    "Keuze van O:  4\n",
    "\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | | |O| | |\n",
    "| | |X|X|O| | |\n",
    "---------------\n",
    " 0 1 2 3 4 5 6\n",
    "\n",
    "Keuze van X:  1\n",
    "\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | | |O| | |\n",
    "| |X|X|X|O| | |\n",
    "---------------\n",
    " 0 1 2 3 4 5 6\n",
    "\n",
    "Keuze van O:  2\n",
    "\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | |O| |O| | |\n",
    "| |X|X|X|O| | |\n",
    "---------------\n",
    " 0 1 2 3 4 5 6\n",
    "\n",
    "Keuze van X:  0\n",
    "\n",
    "\n",
    "X wint -- Gefeliciteerd!\n",
    "\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | | | | | | |\n",
    "| | |O| |O| | |\n",
    "|X|X|X|X|O| | |\n",
    "---------------\n",
    " 0 1 2 3 4 5 6\n",
    "\n",
    "In [3]:\n",
    "```\n"
   ]
  }
 ],
 "metadata": {
  "language_info": {
   "name": "python"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 2
}
